// file ecash.h
#include "reg5000.h"
#include "ecerror.h"
#include "ecow.h"
//#include <stdio.h>
#include <string.h>
#include <ctype.h>

#define FirmwareString "eCash Kit  Rev A"

/*
Explanation of different memory locations recognized by C51:
   code - program memory; accessed by opcode MOVC @A+DPTR (Up to 64 KBytes)
   data - directly addressable internal data memory; fastest access to variables (128 bytes)
   idata - indirectly addressable internal data memory; accessed across the full internal address space (256 bytes)
   bdata - bit-addressable internal data memory; allows mixed bit and byte access (16 bytes)
   xdata - external data memory; accessed by opcode MOVX @DPTR (Up to 64 KBytes)
   pdata - paged external data memory; accessed by opcode MOVX @Rn (256 bytes)

Explanation of different data types and their size for C51:
   char - 1 byte (signed/unsigned)
   enum - 2 bytes
   short - 2 bytes (signed/unsigned)
   int - 2 bytes (signed/unsigned)
   long - 4 bytes (signed/unsigned)
   float - 4 bytes (1.175494E-38 to 3.402823E+38)
   bit - 1 bit
   sbit - 1 bit
   sfr - 1 byte
   sfr16 - 2 bytes
*/

// COMMONLY USED DATA TYPES
#define uchar unsigned char
#define cbyte code unsigned char
// use to declare a bit-addressable byte
#define bbyte bdata unsigned char
// use to declare a scratchpad byte
#define dbyte data unsigned char
// use to declare a scratchpad integer
#define dint data unsigned int
// use to declare an XData byte
#define xbyte xdata unsigned char
// use to declare a bit
#define DECLARE_BIT(byteAddress,bitNumber,bitLabel) sbit bitLabel = byteAddress^bitNumber

// Number of times to retry 1-Wire operations.  0 will do max (256)
#define RetryCount 0

#define DODEBUG 0

#if DODEBUG
   #define DEBUG(str1) serial_putstring(str1, sizeof(str1)-1)
   #define DEBUGLN(str1) DEBUG(str1"\n")
   #define DEBUGBLOCK(block, size) serial_puthexstring(block, size)
   #define DEBUGBLOCKLBL(str1, block, size) {DEBUG(str1); DEBUGBLOCK(block, size); DEBUG("\n");}
#else
   #define DEBUG(str1) //nothing
   #define DEBUGLN(str1) //nothing
   #define DEBUGBLOCK(block, size) //nothing
   #define DEBUGBLOCKLBL(str1, block, size) //nothing
#endif

#define PRINT(str1) serial_putstring(str1, sizeof(str1)-1)
#define PRINTLN(str1) PRINT(str1"\n")
#define PRINTBLOCK(block, size) serial_puthexstring(block, size)
#define PRINTBLOCKLBL(str1, block, size) {PRINT(str1); PRINTBLOCK(block, size); PRINT("\n");}

#define LCD_PRINT(str1) lcd_putstring(str1, sizeof(str1)-1);

#define WAIT_SECS(secs) secTimerRun=0; wait2ms(); TmrSeconds=secs; secTimerRun=1; TimeOut=0; while(!TimeOut);

#define TIMEOUT_VALUE_SECS 15

#define DS5000 5000
#define DS89C420 420
#define PROCESSOR DS89C420

#if PROCESSOR==DS89C420
// the power managment register for the 420
sfr PMR = 0xC4;
sfr EXIF = 0x91;
#endif

/*
; DS5000 (/12) @ 14.7456 = 814 ns cycles
; DS87C520 (/4) @ 14.7456 = 271 ns cycles
; DS89C420 (/1) @ 11.059MHz = 90 ns cycles
; DS89C420 (/1) @ 22.118MHz = 45 ns cycles
; DS89C420 (/1) @ 33.000MHz = 30 ns cycles
*/
#ifndef OSCILLATOR
//   #define OSCILLATOR 33000000
//   #define OSCILLATOR 22118400
//   #define OSCILLATOR 14745600
//   #define OSCILLATOR 14318000
   #define OSCILLATOR 11059200
#endif

// Crystal multiplier for the 420.  By default, the 420 is in 1x mode,
// but it supports both 2x and 4x multipliers.  BUT, the timers are
// still looking at the external clock frequency, so we need to know
// if we are multiplying
//#define CTM2
#ifdef CTM2
   #define CTMDIV 2.0
#else
   #ifdef CTM4
      #define CTMDIV 4.0
   #else
      #define CTMDIV 1.0
   #endif
#endif


// Assumes SMOD=1
#define T1DIV 12.0
#define BAUD9600_RELOAD (256-((2.0*OSCILLATOR/CTMDIV)/(32L*T1DIV*9600)))
#define BAUD19200_RELOAD (256-((2.0*OSCILLATOR/CTMDIV)/(32L*T1DIV*19200)))
#define BAUD57600_RELOAD (256-((2.0*OSCILLATOR/CTMDIV)/(32L*T1DIV*57600)))
#define BAUD115200_RELOAD (256-((2.0*OSCILLATOR/CTMDIV)/(32L*T1DIV*115200)))

#define SInBufSize 128
#define SOutBufSize 128

// Timer 0 reload value for 1kHz ticks
// reload value = 65536 - (OSC/12)/1kHz
// assumes T0DIV=12
#define TIMER0_1MSEC_RELOAD ((int)(65536-((OSCILLATOR/CTMDIV/12.0)/1000L)))


// file ecsearch.c
extern void owNext(void);
extern void owResetSearch(void);

// file ecserial.c
extern void serial_putstring(uchar*, uchar);
extern void serial_puthexstring(uchar*, uchar);
extern void serial_init(void);
extern char putchar(char);

// file ectimer.c
extern void timer0Init(void);
extern void startTimerSec(void);
extern void stopTimerSec(void);
extern void startTimerMS(void);
extern void stopTimerMS(void);
extern void wait2ms(void);
extern void wait10ms(void);
extern void wait250ms(void);
extern void halfUsDelay();
extern void usDelay(unsigned int);
extern void msDelay(unsigned int);

// file crcutil.c
extern void crc16(void);
extern void crc8(void);
extern void crc16_byte(uchar);
extern void crc16_block(uchar*,uchar);
extern void crc8_byte(uchar);
extern void crc_reset(void);

// file ecow.c
extern void ow_init(void);
extern void ow_reset(void);
extern void ow_select(void);
extern void ow_touchbit(void);
extern void ow_touchbyte(void);
extern void ow_touchbyte_crc(void);
extern void ow_touchblock(uchar* block, uchar len);
extern void ow_touchblock_crc(uchar* block, uchar len);
extern void ow_readbyte(void);
extern void ow_readbyte_crc(void);
extern void ow_readblock(uchar*, uchar);
extern void ow_readblock_crc(uchar* block, uchar len);
extern void ow_readRomID(void);

// file ecfuncs.c
extern void selectCoprocessor(void);
extern void accessCoprocessor(void);
extern void selectCustomer(void);
extern void accessCustomer(void);
extern void getDeviceType(void);
extern void prepCoprocessor(void);
extern void loadCoprFile(uchar);
extern void installBindData(void);
extern void eraseScratchpad(void);
extern void verifyCRC(void);
extern void waitUntilDone(void);
extern void pollForToken(void);
extern void getChallenge(void);
extern void issueChallenge(void);
extern void readAccountPage(void);
extern void readToken(void);
extern void formatCertificate(void);
extern void openAccount(void);
extern void fixer(void);
extern void sendNulls(uchar);
extern void computeDeviceSecret(void);
extern void authenticate(void);
extern void setupForSignature(void);
extern void validate(void);
extern void debitCertificate(void);
extern void updateCertificate(void);
extern void writeAccountPage(void);
extern void readScratchpadBack(uchar);
extern void doCopyScratchpd(void);
extern void computeFileCRC();
extern void verifyFile(void);
extern void incPageCounter(void);
extern void makeNewTransID(uchar*);
extern void signCertificate(void);
extern void readBackVerify(void);
extern void waitUntilTokenGone(void);
extern void loadFilePage(uchar);
extern uchar lookupFile(uchar*);
extern void doCopyScratchpad63S(uchar, uchar);
extern void doCopyScratchpad61S(uchar, uchar);
extern void getCopyScratchpadMAC(uchar);
extern void updateAccountPage61S(uchar);

// file ecutil.c
extern void debitBalance(uchar*, uchar*);
extern void beep(uchar);
extern void tick();
extern uchar binToHex(uchar);
extern void byteToHex(uchar, uchar*);
extern uchar hexToBin(uchar);
extern uchar hexToByte(uchar*);
extern uchar balanceToString(void);
extern void getAsciiHexBalance(uchar*, uchar);

// file eclcd.c
extern void lcd_init(void);
extern void lcd_putstring(uchar*, uchar);
extern void lcd_putchar(char);
extern void lcd_clear();
extern void lcd_home();

// file eccmd.c
extern void commandProcessor(void);
extern void getFirmwareVersion(void);
extern void reportError(void);
extern void reportToken(bit);
extern void reportRead(void);
extern void reportDebit(void);
extern void reportReset(void);

// file ecash.c
extern void softReset(void);

// -------------------------------------------------
// seg BITADDR
// -------------------------------------------------
extern bbyte SerialFlags;
extern bit SerialReady; // Set when inbound serial message is ready
extern bit SerialSend; // Set while sending serial message out

extern bbyte ECashFlags;
extern bit NeedAccess; // Set when we need to access the part anew
extern bit DS1963S; // Set when 1963S is customer iButton
extern bit DS1961S; // Set when 1961S is customer iButton
extern bit COPSelected; // Set when Co-Processor is selected
extern bit Copr61S; // Set when 61S coprocessor has been selected
extern bit OWOverdrive; // Set when in OverDrive mode
extern bit SuccessFlag; // Set when a function completes without error
extern bit TokenTear; // Set when Token departs before update/verification completes
//extern bit Mode_ReVal; // Set if re-value is enabled
//extern bit NeedFixer; // Set when '61S "fix" is needed
//extern bit Verbose; // Set for verbose mode

extern bbyte Mode_Flags;
extern bit Mode_AutoPoll; // Auto-poll for any arival
extern bit Mode_AutoRead; // Auto-read and authenticate token after arrival
extern bit Mode_AutoDebit; // Debit token after authentication
extern bit Mode_Pulse; // Make relay pulse on successful debit
extern bit Mode_LCDEnabled; // Set to enable updates to LCD
extern bit Mode_ODEnabled; // Set to enable overdrive mode

extern bbyte Ack_Flags;
extern bit Ack_PollEnabled; // enable lock-step bit for polling
extern bit Ack_PollWaiting; // indicates waiting for acknowledgment of poll
extern bit Ack_ReadEnabled; // enable lock-step bit for reading
extern bit Ack_ReadWaiting; // indicates waiting for acknowledgment of read
extern bit Ack_DebitEnabled; // enable lock-step bit for debiting
extern bit Ack_DebitWaiting; // indicates waiting for acknowledgment of debit

extern bbyte Status_Flags;
extern bit Stat_Poll; // Set when polling is in progress
extern bit Stat_Read; // Set when reading/authenticating is in progress
extern bit Stat_DebitCert; // Set when debit of certificate is in progress
extern bit Stat_UpdateToken; // Set when update of token is in progress
extern bit Stat_Verify; // Set when verify is in progress
extern bit Stat_WaitUntilGone; // Set when waiting for token departure

extern bbyte Timer_Flags;
extern bit TimeOut; // Set when Seconds value has unflowed
extern bit secTimerRun; // Set when TmrSeconds is counting down
extern bit msTimerRun; // Set when msCounter is counting up
extern bit msTimerOverflow; // Set when timer runs for more then 255 ms
extern bit MsgBit; // Message Toggle Flag, set when temp msg is on LCD
extern bit SecBit; // Toggles every Hz

extern bbyte owBuffer; // I/O buffer for all 1-Wire
extern bit owBufferIn; // MSB of owBuffer
extern bit owBufferOut; // LSB of owBuffer

extern bbyte crcl; // low-order byte of CRC, must be bit-addressable

// -------------------------------------------------
// seg DIRECT
// -------------------------------------------------

// Scratchpad memory store definitions -
extern dbyte crch; // High order byte of CRC accumulator
extern dbyte Retry; // 1-Wire Retry counter
extern dbyte DebitAmount[3]; // Debit Amount

// Stores for pages and secrets to be used in debiting -
extern dbyte COP_MAP; // Page used in COP to compute UDS
extern dbyte COP_UDS; // Unique Device Secret re-generated in COP
extern dbyte COP_UDP; // Page used to authenticate cust devices
extern dbyte COP_SDP; // Page used to sign new balances (MUST BE 0 or 8)
extern dbyte CUST_ECP; // Monetary Certificate Page

extern dbyte errorCode;
extern dbyte specificErrorCode;

// temporary variables - intermediate values and counters
//extern dbyte temp1, temp2, temp3;

extern dbyte TmrSeconds; // Counts down at 1Hz
extern dbyte TimeOutValue; // used to set the value for timeouts (in seconds)
extern dbyte msCounter; // Counts up at 1kHz
extern dbyte QtrSeconds; // counts down each quarter second
extern dbyte milliseconds; // counts down millseconds

extern dbyte SInBufPtr; // pointer to last quarter in serial in buffer
extern dbyte SOutBufPtrHead, SOutBufPtrTail; // pointers to data in serial out buffer

extern dbyte waitUntilGoneCounter; // Counts number of resets since token departed

extern dbyte IOPinToPulse; // Stored as portNum/BitMask (i.e. 31 is P3.0, 22 is P2.1)

extern dint Entropy; // entropy used for random transaction ID

// -------------------------------------------------
// seg XRAM
// -------------------------------------------------
#ifndef NOXDATA
extern xbyte LCDMessageDefault[16];
extern xbyte MsgCount; // Counts how long message is shown on LCD


extern xbyte SearchSerialNum[8]; // Rom ID generated from search routine
extern xbyte COP_RomID[8]; // Rom ID of co-processor iButton
extern xbyte COP_RomID_61S[8]; // Rom ID of co-processor iButton used with DS1961S
extern xbyte Cust_RomID[8]; // Rom ID of Customer iButton

extern xbyte OldRecord[32+4];
extern xbyte NewRecord[32+4];
extern xbyte ScratchRecord[32+4];

extern xbyte MaxPrice[2]; // VMC Maximum Price
extern xbyte MinPrice[2]; // VMC Minimum Price (Keep in this order!)

extern xbyte CopySPMAC[20];

extern xbyte RomData[8];
extern xbyte RomData2[8];

extern xbyte BalString[11]; // Space for up to 10 characters plus EOL ("0.00------",0)

extern xbyte ErrString[2]; // Space for a string for an error code

// serial interface buffers
extern xbyte SInBuf[SInBufSize];
extern xbyte SOutBuf[SOutBufSize];

// Image data
extern xbyte BindData[34];
extern xbyte BindCode[7];
extern xbyte SigningChlg[3];
extern xbyte AuthSecret[47];
extern xbyte SignSecret[47];
extern xbyte InitialSig[20];
extern xbyte XSecret[8];
extern xbyte MoneyFileName[5]; // Monetary file name to seek in customer iButton
//ImageSize equ $-BindData
#define ImageSize  168

extern xbyte InitialSigLength;
extern xbyte ProviderNameLength;
extern xbyte ProviderNameBuffer[64];
extern xbyte AuxInfoLength;
extern xbyte AuxInfoBuffer[64];

// eCertificate Definitions

// The eCertificate prototype area is used to build and process monetary values.
// The certificate holds all the data pertinent to the iButton with which a
// monetary transaction is taking place.

// The elements of the eCertificate are defined here. The certificate
// contains a subset of elements that is the record that is stored in the token,
// as well as a subset that is the block sent to SHA-1 for processing.

// Begin the 32 byte token record image -
// New eCert Record Layout -
typedef struct
{
   uchar FileLength; // File: Record Length Byte
   uchar Type; // Certificate Type & Algorithm
   uchar MAC[20]; // Monetary MAC
   uchar MUC[2]; // Monetary Units Code and Multipler
   uchar Balance[3]; // Monetary Balance (Scaled)
   uchar TransID[2]; // Transaction ID
   uchar FileCP; // File: Continuation Pointer
   uchar FileCRC_l; // File: Inverted CRC16
   uchar FileCRC_h; // File: Inverted CRC16
}
StructCertificateRecord;

// struct StructCertificateRecord33
//   Holds user token account data for DS1961S.
//   Usese Double octa-byte scheme for dual money
//   bytes and a pointer to the valid record.
typedef struct
{
   // header
   uchar FileLength;
   uchar Type;
   uchar MUC[2];
   // don't care bytes
   uchar DontCare1[4];
   // record A
   uchar Balance_A[3];
   uchar TransID_A[2];
   uchar FileCP_A;
   uchar FileCRC_l_A;
   uchar FileCRC_h_A;
   // record B
   uchar Balance_B[3];
   uchar TransID_B[2];
   uchar FileCP_B;
   uchar FileCRC_l_B;
   uchar FileCRC_h_B;
   // don't care bytes
   uchar DontCare2[8];

}
StructCertificateRecord33;

// file length used to point at Record A in StructCertificateRecord33
#define RECORD_A_LENGTH 13
// file length used to point at Record B in StructCertificateRecord33
#define RECORD_B_LENGTH 21



//Cert_Object			equ	$ ; // Start of eCertificate Data Object
typedef struct
{
   uchar Descriptor[2]; // Data object descriptor
   uchar ObjectLength; // Data object length

   // 32 byte token record image
   union
   {
      StructCertificateRecord33 rec33;
      StructCertificateRecord rec;
      uchar ptr[32];
   }
   Record;

   uchar WriteCounter[4]; // Write Cycle Counter
   uchar PageNumber; // Token Page Number + X,M bits
   uchar SerialNumber[7]; // Token serial number (no CRC8)
   uchar Challenge[3]; // Challenge Value

   uchar SecretCounter[4]; // Secret write counter
   uchar FileName[4]; // File Name
   uchar FileExt; // File Extension
   uchar DeviceROMID[8]; // Full token Rom ID (w/CRC8)
   //uchar Cert_DeviceChannel; // Device I/O Channel ID
   uchar DevicePage; // Current token page number

   uchar AuthMAC[20]; // Authentication MAC, from Read Auth. Page


// Ancillary Data -

// Associated with each eCertificate is assorted data that we use to access and
// process it -
   //uchar Error; // Last eCert processing error code
}
StructCertificate;

extern xdata StructCertificate Certificate;
#endif